import Flutter
import UIKit
import ContactsUI

@available(iOS 9.0, *)
public class SwiftFluttercontactpickerPlugin: NSObject, FlutterPlugin, CNContactPickerDelegate {
    private var pendingResult: FlutterResult? = nil
    private var pendingResultType = "unknown"
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "fluttercontactpicker", binaryMessenger: registrar.messenger())
        let instance = SwiftFluttercontactpickerPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    private func processContact(result: @escaping FlutterResult, neededProperty: String, type: String) {
        assert(pendingResult == nil, "There is still a pending result")
        pendingResult = result
        pendingResultType = type
        let contactsController = CNContactPickerViewController()
        contactsController.delegate = self
        contactsController.displayedPropertyKeys = [neededProperty]
        let viewController = UIApplication.shared.delegate?.window??.rootViewController
        viewController?.present(contactsController, animated: true, completion: nil)
    }
    
    public func contactPicker(_ picker: CNContactPickerViewController, didSelect contactProperty: CNContactProperty) {
        let fullName = CNContactFormatter.string(from: contactProperty.contact, style: CNContactFormatterStyle.fullName)
        let itemRaw = contactProperty.value
        var item: Any?
        if(itemRaw is CNPhoneNumber) {
            item = (itemRaw as! CNPhoneNumber).stringValue
        } else {
            item = itemRaw
        }
        let label = CNLabeledValue<NSString>.localizedString(forLabel: contactProperty.label!)
        let dict = [
            "fullName": fullName as Any,
            pendingResultType: [
                pendingResultType: item,
                "label": label
            ],
            ] as [String : Any]
        pendingResult!(dict)
        finish()
    }
    
    public func contactPickerDidCancel(_ picker: CNContactPickerViewController) {
        finish()
    }
    
    private func finish() {
        pendingResult = nil
        pendingResultType = "unknown"
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "pickPhoneContact":
            processContact(result: result, neededProperty: CNContactPhoneNumbersKey, type: "phoneNumber")
            break
        case "pickEmailContact":
            processContact(result: result, neededProperty: CNContactEmailAddressesKey, type: "email")
            break
        default:
            result(FlutterMethodNotImplemented)
        }
  }
}
